#!/bin/bash
#
# Simple script to add an MSS domain to Apache configuration.
# Belongs to mss-manager package.
#
# Usage: $0 MSS_name MSS_domain_number domain_hostname

APACHEADMIN="webadmin"                                  # Linux user for web administration
APACHEADMINGROUP="www-data"                             # Linux group for web administration
APACHEAUTHDIR="/etc/apache-ssl/auth.d"                  # Web user's authentication data folder
APACHECONFIGDIR="/etc/apache/conf.d"                    # Directory for apache virtualhosts
APACHEDAEMON="/etc/init.d/apache"                       # Start-stop script for apache
APACHEDOCROOT="/var/www"                                # Public web documentroot
APACHEGROUP="www-data"                                  # Linux group for web content
APACHELOGDIR="/var/log/apache"                          # Root directory for Apache logs
APACHESSLCONFIGDIR="/etc/apache-ssl/conf.d"             # Directory for apache-ssl virtualhosts
APACHESSLDAEMON="/etc/init.d/apache-ssl"                # Start-stop script for apache-ssl
APACHESSLIP="*"                                         # IP for virtualhosts administration
APACHESSLPORT=":443"                                    # SSL service port
APACHESYSTEMCONF="SYSTEM-http.conf"			# Apache main virtualhost configuration
APACHEUPLOADDIR="upload"                                # Special directory for uploader
APACHEUSER="www-data"                                   # Linux user for public webservice
APACHEWEBIP="*"                                         # IP for public web virtualhosts
APACHEWEBPORT=""                                        # HTTP service port
AWK="/usr/bin/awk"					# awk command call
AWSTATSLINK="awstats"                                   # Link name points to web statistics
AWSTATSWEBROOT="$APACHEDOCROOT/awstats"                 # Web statistics html document root
CAT="/bin/cat"						# cat command call
CHOWN="/bin/chown"					# chown command call
DEFAULT_AUDITOR="auditor"                               # Default auditor user (stats)
DEFAULT_EMAIL="root@localhost.localdomain"              # Default email address
DEFAULT_PAGE="index.html"                               # Default webpage
DEFAULT_TOMCAT="system"					# Default Tomcat instance
DEFAULT_WEBMASTER="webmaster"                           # Default webmaster user (DAV)
ENVSUBST="/usr/bin/envsubst"                            # envsubst command call
EXIT_ERR="1"                                            # Exit code on error
EXIT_SUCC="0"                                           # Exit code on success
GREP="/bin/grep"                                        # grep command call
MSSTEMPLATEDIR="/usr/local/share/mss-manager/templates"	# Template directory
MSS500TEMPLATE="500.html"				# HTTP 500/503 error page template
MSSHTACCTEMPLATE=".htaccess"				# .htaccess template
MSSHTTPTEMPLATE="http.conf"				# Apache webserver config template
MSSHTTPSTEMPLATE="https.conf"				# Apache-SSL webserver config template
LN="/bin/ln"						# ln command call
MASKUSER="\([0-9a-zA-Z._-]\)\+"                         # SED regexp: valid username
MASKWEB="\(\([0-9a-zA-Z_-]\)\+\.\)\+\([0-9a-zA-Z]\)\+"  # SED regexp: valid website name
MASKEMAIL="$MASKUSER@$MASKWEB"                          # SED regexp: valid e-mail address
SED="/bin/sed"                                          # sed command call
WSM="/usr/local/sbin/website-manager.sh"		# wsm script call

# Messages

MSG_CANNOTCREATE="Can't create "
MSG_CANNOTCREATEUSERS="Can't create Apache users!\n"
MSG_CREATED="Created file or directory: "
MSG_MISSING="Missing file or directory: "
MSG_MISSING_SSLIP="No IP for apache-ssl service!\n"
MSG_MISSING_SYSTEMHOST="Apache system host isn't found!\n"
MSG_MISSING_WEBIP="No IP for apache service!\n"
MSG_USAGE="Usage:   $0 MSS_name MSS_domain_number domain_hostname\nExample: $0 mss 2 www.example.com\n\n"

# Including headers (if any)
HEADERS="/etc/default/mss-manager"
if [ -f "$HEADERS" ]; then . $HEADERS; fi

# Check templates

if [ ! -f $MSSTEMPLATEDIR/$MSSHTTPTEMPLATE ];  then echo -ne "$MSG_MISSING$MSSTEMPLATEDIR/$MSSHTTPTEMPLATE\n";  exit $EXIT_ERR; fi
if [ ! -f $MSSTEMPLATEDIR/$MSSHTTPSTEMPLATE ]; then echo -ne "$MSG_MISSING$MSSTEMPLATEDIR/$MSSHTTPSTEMPLATE\n"; exit $EXIT_ERR; fi

# Get command-line parameters

MSSNAME=$1; shift
if [ -z "$MSSNAME" ]; then echo -ne "$MSG_USAGE"; exit $EXIT_ERR >&2; fi

MSSNUMBER=$1; shift
if [ -z "$MSSNUMBER" ]; then echo -ne "$MSG_USAGE"; exit $EXIT_ERR >&2; fi

DOMAINNAME=$1; shift
if [ -z "$DOMAINNAME" ]; then echo -ne "$MSG_USAGE"; exit $EXIT_ERR >&2; fi

# Check corresponding Apache configuration

if [ ! -f $APACHECONFIGDIR/$MSSNAME ]; then 
    echo -ne "$MSG_CREATED$APACHECONFIGDIR/$MSSNAME\n"
    touch $APACHECONFIGDIR/$MSSNAME
fi
if [ ! -f $APACHESSLCONFIGDIR/$MSSNAME ]; then 
    echo -ne "$MSG_CREATED$APACHESSLCONFIGDIR/$MSSNAME\n"
    touch $APACHESSLCONFIGDIR/$MSSNAME
fi

# Get various parameters from hosting machine and MSS instance

if [ ! -f $APACHECONFIGDIR/$APACHESYSTEMCONF ]; then echo -ne "$MSG_MISSING$APACHECONFIGDIR/$APACHESYSTEMCONF\n"; exit $EXIT_ERR; fi
parameter=`$CAT $APACHECONFIGDIR/$APACHESYSTEMCONF | $GREP -i -m 1 "servername" | \
           $AWK '{ print $2 }'`
if [ -z "$parameter" ]; then echo -ne "$MSG_MISSING_SYSTEMHOST"; exit $EXIT_ERR >&2; fi
systemhost=$parameter

parameter=`$CAT $APACHECONFIGDIR/$MSSNAME | $GREP -i -m 1 "virtualhost" | \
           $SED -n "s/^.*\([[:space:]]\+\)\(\(\([[:digit:]]\{1,3\}\)\.\)\{3\}[[:digit:]]\{1,3\}\)[^0-9].*$/\2/p"`
APACHEWEBIP=${parameter:-$APACHEWEBIP}
if [ -z "$APACHEWEBIP" ];  then echo -ne "$MSG_MISSING_WEBIP"; exit $EXIT_ERR >&2; fi

parameter=`$CAT $APACHECONFIGDIR/$MSSNAME | $GREP -i -m 1 "virtualhost" | \
           $SED -n "s/^.*\(\:[[:digit:]]\{1,5\}\)[^0-9].*$/\1/p"`
APACHEWEBPORT=${parameter:-$APACHEWEBPORT}

parameter=`$CAT $APACHESSLCONFIGDIR/$MSSNAME | $GREP -i -m 1 "virtualhost" | \
           $SED -n "s/^.*\([[:space:]]\+\)\(\(\([[:digit:]]\{1,3\}\)\.\)\{3\}[[:digit:]]\{1,3\}\)[^0-9].*$/\2/p"`
APACHESSLIP=${parameter:-$APACHESSLIP}
if [ -z "$APACHESSLIP" ];  then echo -ne "$MSG_MISSING_SSLIP"; exit $EXIT_ERR >&2; fi

parameter=`$CAT $APACHESSLCONFIGDIR/$MSSNAME | $GREP -i -m 1 "virtualhost" | \
           $SED -n "s/^.*\(\:[[:digit:]]\{1,5\}\)[^0-9].*$/\1/p"`
APACHESSLPORT=${parameter:-$APACHESSLPORT}

parameter=`$CAT $APACHECONFIGDIR/$MSSNAME | $GREP -i -m 1 "serveradmin" | \
           $AWK '{ print $2 }'`
email=${parameter:-$DEFAULT_EMAIL}

parameter=`$CAT $APACHECONFIGDIR/$MSSNAME | $GREP -i -m 1 "jkmount" | \
           $AWK '{ print $3 }'`
tomcatname=${parameter:-$DEFAULT_TOMCAT}

vhost=$DOMAINNAME

# Apache webserver configuration

DATE=`date '+%Y-%m-%d %H:%M'`
export APACHEDOCROOT APACHELOGDIR APACHEUPLOADDIR APACHEWEBIP APACHEWEBPORT \
       DATE MSSNAME MSSNUMBER email systemhost tomcatname vhost; \
$CAT $MSSTEMPLATEDIR/$MSSHTTPTEMPLATE | $ENVSUBST >>$APACHECONFIGDIR/$DOMAINNAME 
$APACHEDAEMON reload

# Apache SSL webserver configuration

DATE=`date '+%Y-%m-%d %H:%M'`
export APACHEAUTHDIR APACHEDOCROOT APACHELOGDIR APACHEUPLOADDIR APACHESSLIP APACHESSLPORT \
       DATE MSSNAME MSSNUMBER email systemhost vhost; \
$CAT $MSSTEMPLATEDIR/$MSSHTTPSTEMPLATE | $ENVSUBST >>$APACHESSLCONFIGDIR/$DOMAINNAME 
$APACHESSLDAEMON reload

# Creating Apache users and groups

if [ -x $WSM ]; then
    $WSM -au  $DOMAINNAME auditor >&2
    $WSM -ag  $DOMAINNAME stats
    $WSM -aug $DOMAINNAME auditor stats
    $WSM -au  $DOMAINNAME webmaster >&2
    $WSM -ag  $DOMAINNAME admin
    $WSM -aug $DOMAINNAME webmaster admin
else
    echo -ne "$MSG_MISSING $WSM\n" >&2
    echo -ne "$MSG_CANNOTCREATEUSERS" >&2
fi

# Creating extra (per-domain) files

if [ -d $APACHEDOCROOT/$MSSNAME/domain$MSSNUMBER ]; then

# AwStat's link

    if [ ! -L $APACHEDOCROOT/$MSSNAME/domain$MSSNUMBER/$AWSTATSLINK ]; then
	$LN -s $AWSTATSWEBROOT/$vhost $APACHEDOCROOT/$MSSNAME/domain$MSSNUMBER/$AWSTATSLINK
	$CHOWN -h $APACHEADMIN:$APACHEADMINGROUP $APACHEDOCROOT/$MSSNAME/domain$MSSNUMBER/$AWSTATSLINK
	echo -ne "$MSG_CREATED$APACHEDOCROOT/$MSSNAME/domain$MSSNUMBER/$AWSTATSLINK \n"
    fi

# Custom 500/503 http error page

    if [ ! -f $APACHEDOCROOT/$MSSNAME/domain$MSSNUMBER/$MSS500TEMPLATE ]; then
	export email; \
	$CAT $MSSTEMPLATEDIR/$MSS500TEMPLATE | $ENVSUBST >>$APACHEDOCROOT/$MSSNAME/domain$MSSNUMBER/$MSS500TEMPLATE
	$CHOWN $APACHEADMIN:$APACHEADMINGROUP $APACHEDOCROOT/$MSSNAME/domain$MSSNUMBER/$MSS500TEMPLATE
    fi

# Custom .htaccess file

    if [ ! -f $APACHEDOCROOT/$MSSNAME/domain$MSSNUMBER/$MSSHTACCTEMPLATE ]; then
	export MSSNAME MSSNUMBER vhost; \
	$CAT $MSSTEMPLATEDIR/$MSSHTACCTEMPLATE | $ENVSUBST >>$APACHEDOCROOT/$MSSNAME/domain$MSSNUMBER/$MSSHTACCTEMPLATE
	$CHOWN $APACHEADMIN:$APACHEADMINGROUP $APACHEDOCROOT/$MSSNAME/domain$MSSNUMBER/$MSSHTACCTEMPLATE
    fi

else
    echo -ne "$MSG_MISSING$APACHEDOCROOT/$MSSNAME/domain$MSSNUMBER\n" >&2
    echo -ne "$MSG_CANNOTCREATE$APACHEDOCROOT/$MSSNAME/domain$MSSNUMBER/$AWSTATSLINK\n" >&2
    echo -ne "$MSG_CANNOTCREATE$APACHEDOCROOT/$MSSNAME/domain$MSSNUMBER/$MSS500TEMPLATE\n" >&2
    echo -ne "$MSG_CANNOTCREATE$APACHEDOCROOT/$MSSNAME/domain$MSSNUMBER/$MSSHTACCTEMPLATE\n" >&2
fi

# That's all, folks! :-)
